library(akima)
source("lobe.R")

dopmap <- function(file,...) {
  d <- read.table(file,skip=1)
  imgd <- sqrt(nrow(d))
  map <- matrix(d$V3,ncol=imgd,nrow=imgd)
  vmin <- d[1,1]
  par(pty="s")
  image(seq(vmin,-vmin,length.out=imgd),seq(vmin,-vmin,length.out=imgd),map,col=rw.colors(512),xlab="Vx (km/s)",ylab="Vy (km/s)",...)
  cat(sprintf("min=%f, max=%f \n",min(map),max(map)))
}

mkcolbar <- function(x,dy) {
  y <- seq(x[1,3]-dy,x[1,3]+dy,0.01)
  xx <- cbind(rep(x[,1],times=length(y)),rep(y,each=length(x[,1])),rep(x[,2],times=length(y)))
  xx.li <- interp(xx[,1],xx[,2]*100,xx[,3])
  return(xx.li)
}

trail <- function(file,flag,dy,phflag=0,...) {
  data <- read.table(file,skip=1)
  names(data) <- c("v","sp0","sp1","ph")
  cat("Input data: data$v, data$sp0, data$sp1, data$ph \n")

  phase <- unique(data$ph)
  nph <- length(phase)

  if ( phflag ) {
     c <- rep(0,nph)
     c0 <- 0
     for (i in 2:nph) {
     	 if (phase[i] < phase[i-1])
     	 c0 <- c0 + 1
  	 c[i] <- c0
     }
     phase <- phase + c
     c <- rep(0,length(data$v))
     c0 <- 0
     for (i in 2:length(data$v)) {
     	 if (data$ph[i] < data$ph[i-1])
      	    c0 <- c0 + 1
    	 c[i] <- c0
     }
     data$ph <- data$ph + c
  }

  vmin = min(data$v)
  vmax = max(data$v)

  difmin <- min(data$sp1-data$sp0)
  difmax <- max(data$sp1-data$sp0)

  rms <- sqrt(sum((data$sp1 - data$sp0)^2)/length(data$sp1))
  cat(sprintf("Spectra   min=%f, max=%f\n",min(data$sp0),max(data$sp0)))
  cat(sprintf("Residuals min=%f, max=%f rms=%f\n",difmin,difmax,rms))

  oldpar <- par(no.readonly=TRUE)
  for (i in 1:nph) {
    d <- data[which(data$ph==phase[i]),]
    if (flag==1)
      d <- cbind(d$v,d$sp1,d$ph)
    else if (flag==2)
      d <- cbind(d$v,d$sp1-d$sp0,d$ph)
    else
      d <- cbind(d$v,d$sp0,d$ph)
    x <- mkcolbar(as.matrix(d),dy)
    ymin <- min(phase)-dy
    ymax <- max(phase)+dy
    if ( i != 1 ){
      par(pty="s")
      par(new=T)
      par(xaxt="n")
      par(yaxt="n")
      image(x$x,x$y/100,x$z,col=grey(512:0/512),xlim=c(vmin,vmax),ylim=c(ymin,ymax),xlab="",ylab="",...)
    }
  else
    image(x$x,x$y/100,x$z,col=grey(512:0/512),xlim=c(vmin,vmax),ylim=c(ymin,ymax),xlab="Radial velocity (km/s)",ylab="Orbital phase",...)
  }
  par(oldpar)
}

rw.colors <- function (n, bf = 1.0) 
{
    if (n <= 0) 
        return(character(0))
    x <- seq(0, 1, length = n)
    r <- 1/(1 + exp((0.375-x)/0.06))
    g <- 1/(1 + exp((0.70-x)/0.05))
    b <- pmax( bf*dnorm(x, 0.25, 0.15)/max(dnorm(x, 0.25, 0.15)) , 2/(1 + exp((1.0-x)/0.05)) )
    col <- mapply(rgb, r, g, b, 1.0)
    return(col)
}
